#include "stdafx.h"
#include "MapiApp.h"
#include "MapiAppDlg.h"
#include "MapiAppConfigurationDlg.h"
#include "CMapi.h"
#include "EnumMessagesDlg.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#endif


#pragma warning(suppress: 26433 26440)
BEGIN_MESSAGE_MAP(CMailDlg, CDialog) //NOLINT(modernize-avoid-c-arrays)
  ON_WM_PAINT()
  ON_WM_QUERYDRAGICON()
  ON_BN_CLICKED(IDC_CONFIGURATION, &CMailDlg::OnConfiguration)
  ON_BN_CLICKED(IDC_SEND, &CMailDlg::OnSend)
  ON_BN_CLICKED(IDC_BROWSE_FILE, &CMailDlg::OnBrowseFile)
  ON_WM_DESTROY()
  ON_BN_CLICKED(IDC_READ, &CMailDlg::OnRead)
END_MESSAGE_MAP()

#pragma warning(suppress: 26455)
CMailDlg::CMailDlg(_In_opt_ CWnd* pParent) : CDialog{CMailDlg::IDD, pParent},
                                             m_bResolve{FALSE},
                                             m_bInteractive{FALSE},
                                             m_bShared{FALSE},
                                             m_bReadReceipt{FALSE},
                                             m_bHTML{FALSE},
                                             m_hIcon{AfxGetApp()->LoadIcon(IDR_MAINFRAME)}
{
  //Load in the values from the registry
#pragma warning(suppress: 26429)
  CWinApp* pApp{AfxGetApp()};
  ASSERT(pApp != nullptr);
  CString sSection{_T("Settings")};

  //Decrypt the profile
  BYTE* pEncryptedProfile{nullptr};
  UINT nEncryptedProfileLength{0};
  if (pApp->GetProfileBinary(sSection, _T("ProfileE"), &pEncryptedProfile, &nEncryptedProfileLength))
  {
    DATA_BLOB dbIn{};
    dbIn.cbData = nEncryptedProfileLength;
    dbIn.pbData = pEncryptedProfile;
    DATA_BLOB dbOut{};
    if (CryptUnprotectData(&dbIn, nullptr, nullptr, nullptr, nullptr, CRYPTPROTECT_UI_FORBIDDEN, &dbOut))
    {
#pragma warning(suppress: 26490)
      m_sProfile = reinterpret_cast<LPCWSTR>(dbOut.pbData);
      LocalFree(dbOut.pbData);
    }
  }
#pragma warning(suppress: 26401 26409)
  delete [] pEncryptedProfile;

  //Decrypt the password
  BYTE* pEncryptedPassword{nullptr};
  UINT nEncryptedPasswordLength{0};
  if (pApp->GetProfileBinary(sSection, _T("PasswordE"), &pEncryptedPassword, &nEncryptedPasswordLength))
  {
    DATA_BLOB dbIn{};
    dbIn.cbData = nEncryptedPasswordLength;
    dbIn.pbData = pEncryptedPassword;
    DATA_BLOB dbOut{};
    if (CryptUnprotectData(&dbIn, nullptr, nullptr, nullptr, nullptr, CRYPTPROTECT_UI_FORBIDDEN, &dbOut))
    {
#pragma warning(suppress: 26490)
      m_sPassword = reinterpret_cast<LPCWSTR>(dbOut.pbData);
      LocalFree(dbOut.pbData);
    }
  }
#pragma warning(suppress: 26401 26409)
  delete [] pEncryptedPassword;
}

#pragma warning(suppress: 26429)
void CMailDlg::DoDataExchange(CDataExchange* pDX)
{
  __super::DoDataExchange(pDX);
  DDX_Text(pDX, IDC_BCC, m_sBCC);
  DDX_Text(pDX, IDC_BODY, m_sBody);
  DDX_Text(pDX, IDC_CC, m_sCC);
  DDX_Text(pDX, IDC_ATTACHMENTS, m_sFiles);
  DDX_Text(pDX, IDC_SUBJECT, m_sSubject);
  DDX_Text(pDX, IDC_TO, m_sTo);
  DDX_Check(pDX, IDC_RESOLVE, m_bResolve);
  DDX_Check(pDX, IDC_INTERACTIVE, m_bInteractive);
  DDX_Check(pDX, IDC_SHARED, m_bShared);
  DDX_Check(pDX, IDC_READ_RECEIPT, m_bReadReceipt);
  DDX_Check(pDX, IDC_HTML, m_bHTML);
  if (pDX->m_bSaveAndValidate)
  {
    if (m_sTo.IsEmpty() && !m_bInteractive) //Allow an empty To address if we are "Interactive"
    {
      AfxMessageBox(_T("Please specify a valid To: address"), MB_OK | MB_ICONEXCLAMATION);
      pDX->PrepareEditCtrl(IDC_TO);
      pDX->Fail();
    }
  }
}

BOOL CMailDlg::OnInitDialog()
{
  __super::OnInitDialog();
  SetIcon(m_hIcon, TRUE);
  return TRUE;
}

#pragma warning(suppress: 26434)
void CMailDlg::OnPaint()
{
  if (IsIconic())
  {
    CPaintDC dc{this}; //device context for painting

#pragma warning(suppress: 26490)
    SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

    //Center icon in client rectangle
    const int cxIcon{GetSystemMetrics(SM_CXICON)};
    const int cyIcon{GetSystemMetrics(SM_CYICON)};
    CRect rect;
    GetClientRect(&rect);
    const int x{(rect.Width() - cxIcon + 1) / 2};
    const int y{(rect.Height() - cyIcon + 1) / 2};

    //Draw the icon
    dc.DrawIcon(x, y, m_hIcon);
  }
  else
    __super::OnPaint();
}

#pragma warning(suppress: 26434 26440)
HCURSOR CMailDlg::OnQueryDragIcon()
{
#pragma warning(suppress: 26473)
  return static_cast<HCURSOR>(m_hIcon);
}

void CMailDlg::OnConfiguration()
{
  CMailConfigurationDlg dlg;
  dlg.m_sProfile = m_sProfile;
  dlg.m_sPassword = m_sPassword;
  if (dlg.DoModal() == IDOK)
  {
    m_sProfile = dlg.m_sProfile;
    m_sPassword = dlg.m_sPassword;
  }
}

void CMailDlg::OnSend()
{
  if (UpdateData(TRUE))
  {
    //Display a wait cursor
    CWaitCursor wc;

    //Create the message
    CMapi::CMessage message;

    //Setup the all the recipient types for this message
    if (m_sTo.GetLength())
#pragma warning(suppress: 26489)
      CMapi::CMessage::AddMultipleRecipients(CMapi::String{CString{m_sTo}}, message.m_To);
    if (m_sCC.GetLength())
#pragma warning(suppress: 26489)
      CMapi::CMessage::AddMultipleRecipients(CMapi::String{CString{m_sCC}}, message.m_CC);
    if (m_sBCC.GetLength())
#pragma warning(suppress: 26489)
      CMapi::CMessage::AddMultipleRecipients(CMapi::String{CString{m_sBCC}}, message.m_BCC);

    //Add the Subject and Body
    if (!m_sSubject.IsEmpty())
#pragma warning(suppress: 26489)
      message.m_sSubject = m_sSubject;
    if (!m_sBody.IsEmpty())
#pragma warning(suppress: 26489)
      message.m_sBody = m_sBody;

    //Add the attachments if necessary
    if (!m_sFiles.IsEmpty())
      message.AddMultipleAttachments(CMapi::String(m_sFiles));

    //Add the read receipt flag if required
    if (m_bReadReceipt)
      message.m_Flags = MAPI_RECEIPT_REQUESTED;

    //connect to server
    CMapi::CSession connection;
    BOOL bLogon{FALSE};
    if (m_bShared)
      bLogon = connection.SharedLogon();
    else
      bLogon = connection.Logon(std::string{CStringA{m_sProfile}}, std::string{CStringA{m_sPassword}}, GetSafeHwnd());
    if (!bLogon)
    {
      CString sMsg;
      sMsg.Format(_T("Could not login to MAPI!, Error:0x%08X"), connection.GetLastError());
      AfxMessageBox(sMsg, MB_OK | MB_ICONSTOP);
      return;
    }

    //Send the message
#pragma warning(suppress: 26486)
    if (!connection.Send(message, m_bResolve, m_bInteractive, m_bInteractive ? GetSafeHwnd() : nullptr, m_bHTML))
    {
      //Dont bother reporting the error MAPI_E_USER_ABORT if we were interactive
      const ULONG nMAPIError{connection.GetLastError()};
      if (!(m_bInteractive && (nMAPIError == MAPI_E_USER_ABORT)))
      {
        CString sMsg;
        sMsg.Format(_T("Could not send message!, Error:0x%08X"), connection.GetLastError());
        AfxMessageBox(sMsg, MB_OK | MB_ICONSTOP);
      }
    }
  }
}

void CMailDlg::OnBrowseFile()
{
  //Bring up a standard browse for file dialog
  CFileDialog dlg{TRUE, nullptr, nullptr, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, _T("All Files (*.*)|*.*||")};
  if (dlg.DoModal() == IDOK)
  {
    CString sAttachments;
    GetDlgItemText(IDC_ATTACHMENTS, sAttachments);

    if (sAttachments.IsEmpty())
      sAttachments = dlg.GetPathName();
    else
    {
      sAttachments += _T(", ");
      sAttachments += dlg.GetPathName();
    }
    SetDlgItemText(IDC_ATTACHMENTS, sAttachments);
  }
}

#pragma warning(suppress: 26434)
void CMailDlg::OnDestroy()
{
  __super::OnDestroy();

  //Save in the values from the registry
#pragma warning(suppress: 26429)
  CWinApp* pApp{AfxGetApp()};
  ASSERT(pApp != nullptr);
  CString sSection{_T("Settings")};

  //Encrypt the profile
  CStringW sTempProfile{m_sProfile};
  DATA_BLOB dbIn{};
  const int nProfileLength{sTempProfile.GetLength()};
  dbIn.cbData = (nProfileLength + 1) * sizeof(wchar_t);
#pragma warning(suppress: 26490)
  dbIn.pbData = reinterpret_cast<BYTE*>(sTempProfile.GetBuffer(nProfileLength));
  DATA_BLOB dbOut{};
  if (CryptProtectData(&dbIn, nullptr, nullptr, nullptr, nullptr, CRYPTPROTECT_UI_FORBIDDEN, &dbOut))
  {
    pApp->WriteProfileBinary(sSection, _T("ProfileE"), dbOut.pbData, dbOut.cbData);
    LocalFree(dbOut.pbData);
  }
  sTempProfile.ReleaseBuffer();

  //Encrypt the password
  CStringW sTempPassword{m_sPassword};
  const int nPasswordLength{sTempPassword.GetLength()};
  dbIn.cbData = (nPasswordLength + 1) * sizeof(wchar_t);
#pragma warning(suppress: 26490)
  dbIn.pbData = reinterpret_cast<BYTE*>(sTempPassword.GetBuffer(nPasswordLength));
  if (CryptProtectData(&dbIn, nullptr, nullptr, nullptr, nullptr, CRYPTPROTECT_UI_FORBIDDEN, &dbOut))
  {
    pApp->WriteProfileBinary(sSection, _T("PasswordE"), dbOut.pbData, dbOut.cbData);
    LocalFree(dbOut.pbData);
  }
  sTempPassword.ReleaseBuffer();
}

void CMailDlg::OnRead()
{
  CMapi::CSession connection;
  BOOL bLogon{FALSE};
  if (m_bShared)
    bLogon = connection.SharedLogon();
  else
    bLogon = connection.Logon(std::string{CStringA{m_sProfile}}, std::string{CStringA{m_sPassword}}, GetSafeHwnd());
  if (!bLogon)
  {
    CString sMsg;
    sMsg.Format(_T("Couldn't login to MAPI!, Error:0x%08X"), connection.GetLastError());
    AfxMessageBox(sMsg, MB_ICONSTOP);
  }
  else
  {
    //Bring up the Enum Messages dialog
    CEnumMessagesDlg dlg;
    dlg.m_pMAPISession = &connection;
    dlg.DoModal();
  }
}
